export declare enum WarningTypes {
    WARNING = "Warning",
    DEPRECATION = "DeprecationWarning"
}
export declare function warn(warning: Warning): void;
export interface Warning {
    code: string;
    type: WarningTypes;
    message: string;
    warned?: boolean;
}
