import * as protobuf from 'protobufjs';
import { FromObjectValue } from './types';
export interface Duration {
    seconds: number;
    nanos?: number;
}
export declare function googleProtobufDurationToProto3JSON(obj: protobuf.Message & Duration): string;
export declare function googleProtobufDurationFromProto3JSON(json: string): {
    [key: string]: FromObjectValue;
};
