export declare const SymbolObj: any;
export declare const ReflectObj: any;
export declare const __hasSymbol: boolean;
export declare const __hasReflect: boolean;
export declare type ObjAssignFunc = (t: any, ...sources: any[]) => any;
export declare var __objAssignFnImpl: ObjAssignFunc;
export declare var __assignFn: ObjAssignFunc;
export declare function __extendsFn(d: any, b: any): void;
export declare function __restFn(s: any, e: any): {};
export declare function __decorateFn(decorators: any, target: any, key: any, desc: any): any;
export declare function __paramFn(paramIndex: number, decorator: Function): (target: any, key: any) => void;
export declare function __metadataFn(metadataKey: any, metadataValue: any): any;
export declare function __exportStarFn(m: any, o: any): void;
export declare function __createBindingFn(o: any, m: any, k: any, k2?: any): void;
export declare function __valuesFn(o: any): any;
export declare function __readFn(o: any, n: any): any;
/** @deprecated */
export declare function __spreadArraysFn(): any[];
export declare function __spreadArrayFn(to: any, from: any): any;
export declare function __makeTemplateObjectFn(cooked: any, raw: any): any;
export declare function __importStarFn(mod: any): any;
export declare function __importDefaultFn(mod: any): any;
